(* File: point_basics.ml

   Copyright (C) 2011-2015

     Fabian Pijcke <fabian.pijcke@gmail.com>
     Pierre Hauweele <antegallya@gmail.com>

   This library is free software; you can redistribute it and/or modify
   it under the terms of the GNU Lesser General Public License version 3 or
   later as published by the Free Software Foundation, with the special
   exception on linking described in the file LICENSE.

   This library is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the file
   LICENSE for more details. *)

open Printf
open OUnit
open Common

open Point

let tests = "Point basic tests" >::: [
  "Poles" >:: (fun () ->
    let p1 = Spherical (0., 0.) in
    let p2 = Spherical (pi, 0.) in
    "Distance is not correct" @? (cmp_float 2. (distance p1 p2))
  );

  "From Spherical to Cartesian" >:: (fun () ->
    let p1 = Spherical (1.42, 2.0492) in
    let p2 = to_cartesian p1 in
    "Points are not equal after conversion" @? (p1 =* p2)
  );

  "From Cartesian to Spherical" >:: (fun () ->
    (* p1 has to be on the unit sphere *)
    let x = 0.438 and y = -0.4928 in
    let z = sqrt (1. -. x *. x -. y *. y) in
    let p1 = Cartesian (x, y, z) in
    let p2 = to_spherical p1 in
    "Points are not equal after conversion" @? (p1 =* p2)
  );

  "Random spherical points always on the unit sphere" >:: (fun () ->
    Random.self_init ();
    for i = 0 to 10000 do
      let theta = Random.float pi and phi = Random.float (2. *. pi) in
      let p = Spherical (theta, phi) in
      let origin = Cartesian (0., 0., 0.) in
      "Not on the unit sphere" @? (distance origin p =~ 1.)
    done
  );
]

